@@ -8,7 +8,7 @@
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::Value;
-$Config::Model::Value::VERSION = '2.123';
+$Config::Model::Value::VERSION = '2.124';
 use 5.10.1;
 
 use Mouse;
@@ -1759,20 +1759,28 @@ sub fetch {
 
 sub map_write_as {
     my $self = shift;
-    my @res = map {
-        my $do_map = (defined $_ and $self->{write_as} and $self->value_type eq 'boolean');
-        $do_map ? $self->{write_as}[$_] : $_;
-    } @_ ;
+    my @res;
+    if ($self->{write_as} and $self->value_type eq 'boolean') {
+        foreach my $v (@_) {
+            push @res, ( defined $v and $v =~ /^\d+$/ ) ? $self->{write_as}[$v] : $v;
+        }
+    }
+    else {
+        @res = @_;
+    }
     return wantarray ? @res : $res[0];
 }
 
-# modifies in place the passed values (which are also returned)
+# modifies in place the passed values
 sub map_write_as_inplace {
     my $self = shift;
-    map {
-        $_ = $self->{write_as}[$_]
-            if defined $_ and $self->{write_as} and $self->value_type eq 'boolean';
-    } @_;
+    return unless $self->{write_as} and $self->value_type eq 'boolean';
+
+    foreach (@_) {
+        if (defined $_ and /^\d+$/) {
+            $_ = $self->{write_as}[$_];
+        }
+    }
 }
 
 sub user_value {
@@ -1878,7 +1886,7 @@ Config::Model::Value - Strongly typed configuration value
 
 =head1 VERSION
 
-version 2.123
+version 2.124
 
 =head1 SYNOPSIS
 
