@@ -8,7 +8,7 @@
 #   The GNU Lesser General Public License, Version 2.1, February 1999
 #
 package Config::Model::WarpedNode;
-$Config::Model::WarpedNode::VERSION = '2.123';
+$Config::Model::WarpedNode::VERSION = '2.124';
 use Mouse;
 
 use Carp qw(cluck croak);
@@ -198,11 +198,11 @@ sub set_properties {
 
     # bringing a new object does not really modify the content of the config tree.
     # only changes underneath changes the tree. And these changes below triggers
-    # their own change notif. SO there's no need to call notify_change when transitioning
+    # their own change notif. So there's no need to call notify_change when transitioning
     # from an undef object into a real object. On the other hand, warping out an object does
     # NOT trigger notify_changes from below. So notify_change must be called
-    if ( defined $old_object ) {
-        my $from = $old_config_class_name // '<undef>';
+    if ( defined $old_object and $old_config_class_name) {
+        my $from = $old_config_class_name ;
         my $to   = $config_class_name     // '<undef>';
         $self->notify_change( note => "warped node from $from to $to" );
     }
@@ -313,7 +313,7 @@ Config::Model::WarpedNode - Node that change config class properties
 
 =head1 VERSION
 
-version 2.123
+version 2.124
 
 =head1 SYNOPSIS
 
